# Generated by Django 2.2.17 on 2021-03-10 16:23

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ExternalStory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.TextField()),
                ('url_hash', models.CharField(db_index=True, editable=False, max_length=32, unique=True)),
                ('title', models.TextField(blank=True, editable=False)),
                ('publisher', models.TextField(editable=False)),
                ('publisher_logo_src', models.TextField(blank=True, editable=False, verbose_name='Publisher logo URL')),
                ('poster_portrait_src', models.TextField(blank=True, editable=False, verbose_name='Poster portrait image URL')),
                ('poster_square_src', models.TextField(blank=True, editable=False, verbose_name='Poster square image URL')),
                ('poster_landscape_src', models.TextField(blank=True, editable=False, verbose_name='Poster landscape image URL')),
                ('last_fetched_at', models.DateTimeField()),
            ],
        ),
    ]



from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _
import requests
from wagtail.core import blocks
from webstories import Story, StoryPage

from .markup import AMPText


class AMPCleanHTMLBlock(blocks.RawHTMLBlock):
    def clean(self, value):
        if isinstance(value, AMPText) and getattr(settings, 'WAGTAIL_WEBSTORIES_CLEAN_HTML', True):
            return AMPText(StoryPage.clean_html_fragment(value.source))
        else:
            return value

    def get_default(self):
        if isinstance(self.meta.default, AMPText):
            return self.meta.default
        else:
            return AMPText(self.meta.default)

    def to_python(self, value):
        if isinstance(value, AMPText):
            return value
        else:
            return AMPText(value)

    def get_prep_value(self, value):
        if isinstance(value, AMPText):
            return value.source
        else:
            return value

    def value_for_form(self, value):
        if isinstance(value, AMPText):
            return value.source
        else:
            return value

    def value_from_form(self, value):
        return AMPText(value)


class PageBlock(blocks.StructBlock):
    id = blocks.CharBlock()
    html = AMPCleanHTMLBlock()


class StoryChooserBlock(blocks.PageChooserBlock):
    def __init__(self, **kwargs):
        has_specified_page_type = kwargs.get('page_type') or kwargs.get('target_model')
        if not has_specified_page_type:
            # allow selecting any page model that inherits from BaseWebStoryPage
            from .models import get_story_page_models
            kwargs['target_model'] = get_story_page_models()

        super().__init__(**kwargs)

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        context['page'] = value.specific
        return context

    class Meta:
        template = 'wagtail_webstories/blocks/story_poster_link.html'


class StoryEmbedBlock(StoryChooserBlock):
    class Meta:
        template = 'wagtail_webstories/blocks/story_embed_block.html'


class ExternalStoryBlock(blocks.URLBlock):
    def get_default(self):
        from .models import ExternalStory
        # Allow specifying the default as either an ExternalStory or a URL string (or None).
        if not self.meta.default:
            return None
        elif isinstance(self.meta.default, ExternalStory):
            return self.meta.default
        else:
            # assume default has been passed as a string
            return ExternalStory.get_for_url(self.meta.default)

    def to_python(self, value):
        from .models import ExternalStory
        # The JSON representation of an ExternalStoryBlock value is a URL string;
        # this should be converted to an ExternalStory instance (or None).
        if not value:
            return None
        else:
            return ExternalStory.get_for_url(value)

    def get_prep_value(self, value):
        # serialisable value should be a URL string
        if value is None:
            return ''
        else:
            return value.url

    def value_for_form(self, value):
        # the value to be handled by the URLField is a plain URL string (or the empty string)
        if value is None:
            return ''
        elif isinstance(value, str):
            return value
        else:
            return value.url

    def value_from_form(self, value):
        # Keep value as a string, and convert to an ExternalStory during clean
        return value or None

    def clean(self, value):
        from .models import ExternalStory
        value = super().clean(value)

        if value is not None:
            try:
                value = ExternalStory.get_for_url(value)
            except requests.exceptions.RequestException:
                raise ValidationError(_("Could not fetch URL."))
            except Story.InvalidStoryException:
                raise ValidationError(_("URL is not a valid web story."))

        return value

    def get_context(self, value, parent_context=None):
        context = super().get_context(value, parent_context=parent_context)
        context['story'] = value
        return context

    class Meta:
        template = 'wagtail_webstories/blocks/external_story_poster_link.html'


class ExternalStoryEmbedBlock(ExternalStoryBlock):
    class Meta:
        template = 'wagtail_webstories/blocks/external_story_embed_block.html'



from urllib.parse import urljoin

from django.apps import apps
from django.conf import settings
from django.template.response import TemplateResponse
from django.shortcuts import redirect
from django.utils.translation import gettext as _
import requests
from wagtail.admin import messages
from webstories import Story

from .forms import ImportStoryForm
from .markup import AMPText

def import_story(request):
    if request.method == 'POST':
        form = ImportStoryForm(request.POST, user=request.user)
        if form.is_valid():
            source_url = form.cleaned_data['source_url']
            try:
                req = requests.get(source_url)
                req.encoding = 'utf-8'
                html = req.text
                story = Story(html)
                story_is_valid = True
            except requests.exceptions.RequestException:
                form.add_error('source_url', _("Could not fetch URL."))
                story_is_valid = False
            except Story.InvalidStoryException:
                form.add_error('source_url', _("URL is not a valid web story."))
                story_is_valid = False

            if story_is_valid:
                page_model = apps.get_model(settings.WAGTAIL_WEBSTORIES_IMPORT_MODEL)

                if story.publisher_logo_src:
                    publisher_logo_src = urljoin(source_url, story.publisher_logo_src)
                else:
                    publisher_logo_src = ''

                if story.poster_portrait_src:
                    poster_portrait_src = urljoin(source_url, story.poster_portrait_src)
                else:
                    poster_portrait_src = ''

                if story.poster_square_src:
                    poster_square_src = urljoin(source_url, story.poster_square_src)
                else:
                    poster_square_src = ''

                if story.poster_landscape_src:
                    poster_landscape_src = urljoin(source_url, story.poster_landscape_src)
                else:
                    poster_landscape_src = ''

                page = page_model(
                    title=story.title,
                    publisher=story.publisher or '',
                    publisher_logo_src_original=publisher_logo_src,
                    poster_portrait_src_original=poster_portrait_src,
                    poster_square_src_original=poster_square_src,
                    poster_landscape_src_original=poster_landscape_src,
                    custom_css=story.custom_css or '',
                    original_url=source_url,
                )
                if getattr(settings, 'WAGTAIL_WEBSTORIES_CLEAN_HTML', True):
                    page.pages = [
                        ('page', {'id': subpage.id, 'html': AMPText(subpage.get_clean_html())})
                        for subpage in story.pages
                    ]
                else:
                    page.pages = [
                        ('page', {'id': subpage.id, 'html': AMPText(subpage.html)})
                        for subpage in story.pages
                    ]

                parent_page = form.cleaned_data['destination']
                parent_page.add_child(instance=page)
                messages.success(request, _("Story '%s' imported.") % page.title)
                return redirect('wagtailadmin_explore', parent_page.id)
    else:
        form = ImportStoryForm(user=request.user)

    return TemplateResponse(request, 'wagtail_webstories/admin/import.html', {
        'form': form,
    })



import hashlib
import json
import os.path
from urllib.parse import urljoin, urlparse

from bs4 import BeautifulSoup
from django.apps import apps
from django.core.exceptions import ValidationError
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
import requests
from wagtail.admin.edit_handlers import FieldPanel, MultiFieldPanel, StreamFieldPanel
from wagtail.core.fields import StreamField
from django.core.files.base import ContentFile, File
from django.core.files.images import ImageFile
from wagtail.core.models import Page, get_page_models
from wagtail.images import get_image_model_string
from wagtail.images.edit_handlers import ImageChooserPanel
from wagtail.images.models import Filter
from webstories import Story

from .blocks import PageBlock
from .markup import AMPText


# Retrieve the (possibly custom) image model. Can't use get_image_model as of Wagtail 2.11, as we
# need require_ready=False: https://github.com/wagtail/wagtail/pull/6568
Image = apps.get_model(get_image_model_string(), require_ready=False)


def _name_from_url(url):
    url_obj = urlparse(url)
    filename = url_obj.path.split('/')[-1]
    return os.path.splitext(filename)[0]


class WebStoryPageMixin(models.Model):
    PUBLISHER_LOGO_IMAGE_FILTER = 'original'
    PORTRAIT_IMAGE_FILTER = 'fill-640x853'
    SQUARE_IMAGE_FILTER = 'fill-640x640'
    LANDSCAPE_IMAGE_FILTER = 'fill-853x640'

    publisher = models.CharField(blank=False, max_length=2047)

    publisher_logo = models.ForeignKey(Image, blank=True, null=True, related_name='+', on_delete=models.SET_NULL)
    publisher_logo_src_original = models.URLField('Publisher logo URL', blank=True, max_length=2047, editable=False)

    poster_image = models.ForeignKey(Image, blank=True, null=True, related_name='+', on_delete=models.SET_NULL)
    poster_portrait_src_original = models.URLField('Poster portrait image URL', blank=True, max_length=2047, editable=False)
    poster_square_src_original = models.URLField('Poster square image URL', blank=True, max_length=2047, editable=False)
    poster_landscape_src_original = models.URLField('Poster landscape image URL', blank=True, max_length=2047, editable=False)

    original_url = models.URLField('Original URL', blank=True, max_length=2047)

    custom_css = models.TextField(blank=True)

    pages = StreamField([
        ('page', PageBlock()),
    ])

    web_story_content_panels = [
        FieldPanel('custom_css'),
        StreamFieldPanel('pages'),
    ]

    web_story_promote_panels = [
        MultiFieldPanel([
            FieldPanel('publisher'),
            ImageChooserPanel('publisher_logo'),
            FieldPanel('original_url'),
        ], heading="Publisher"),
        MultiFieldPanel([
            ImageChooserPanel('poster_image'),
        ], heading="Poster"),
    ]

    def clean(self):
        super().clean()

        errors = {}
        if not (self.publisher_logo or self.publisher_logo_src_original):
            errors['publisher_logo'] = _("A publisher logo must be provided.")

        if not (self.poster_image or self.poster_portrait_src_original):
            errors['poster_image'] = _("A poster image must be provided.")

        if errors:
            raise ValidationError(errors)

    def get_publisher_logo_rendition(self):
        if self.publisher_logo:
            return self.publisher_logo.get_rendition(self.PUBLISHER_LOGO_IMAGE_FILTER)

    @property
    def publisher_logo_src(self):
        rendition = self.get_publisher_logo_rendition()
        if rendition:
            return rendition.url
        else:
            return urljoin(self.original_url, self.publisher_logo_src_original)

    def get_poster_portrait_rendition(self):
        if self.poster_image:
            return self.poster_image.get_rendition(self.PORTRAIT_IMAGE_FILTER)

    @property
    def poster_portrait_src(self):
        rendition = self.get_poster_portrait_rendition()
        if rendition:
            return rendition.url
        else:
            return urljoin(self.original_url, self.poster_portrait_src_original)

    def get_poster_square_rendition(self):
        if self.poster_image:
            return self.poster_image.get_rendition(self.SQUARE_IMAGE_FILTER)

    @property
    def poster_square_src(self):
        rendition = self.get_poster_square_rendition()
        if rendition:
            return rendition.url
        else:
            return urljoin(self.original_url, self.poster_square_src_original)

    def get_poster_landscape_rendition(self):
        if self.poster_image:
            return self.poster_image.get_rendition(self.LANDSCAPE_IMAGE_FILTER)

    @property
    def poster_landscape_src(self):
        rendition = self.get_poster_landscape_rendition()
        if rendition:
            return rendition.url
        else:
            return urljoin(self.original_url, self.poster_landscape_src_original)

    @property
    def linked_data(self):
        return {
            "@context": "http://schema.org",
            "@type": "NewsArticle",
            "mainEntityOfPage": {
                "@type": "WebPage",
                "@id": self.full_url,
            },
            "headline": self.title,
            "image": list(filter(bool, [
                self.poster_portrait_src, self.poster_square_src, self.poster_landscape_src
            ])),
            "datePublished": (
                self.first_published_at and self.first_published_at.isoformat()
            ),
            "dateModified": (
                self.last_published_at and self.last_published_at.isoformat()
            ),
            "publisher": {
                "@type": "Organisation",
                "name": self.publisher,
                "logo": {
                    "@type": "ImageObject",
                    "url": self.publisher_logo_src
                }
            }
        }

    def get_context(self, request):
        context = super().get_context(request)
        context['ld_json'] = json.dumps(self.linked_data)
        return context

    def import_images(self):
        # if flag indicates we have imported images on this instance already,
        # don't repeat; this allows us to call import_images / save within a
        # post_save signal without the second save retriggering a full import_images
        if getattr(self, '_has_imported_images', False):
            return False  # report no changes

        meta_has_changed = self._import_metadata_images()
        content_has_changed = self._import_content_images()
        self._has_imported_images = True
        return meta_has_changed or content_has_changed

    def _import_metadata_images(self):
        has_changed = False

        if self.publisher_logo_src_original and not self.publisher_logo:
            try:
                self.publisher_logo, created = self._image_from_url(
                    urljoin(self.original_url, self.publisher_logo_src_original),
                    title="%s logo" % self.publisher,
                )
                has_changed = True
            except requests.exceptions.RequestException:
                pass

        if self.poster_portrait_src_original and not self.poster_image:
            try:
                portrait_image_file = self._image_file_from_url(
                    urljoin(self.original_url, self.poster_portrait_src_original)
                )
                self.poster_image, created = self._image_from_image_file(
                    portrait_image_file, title=self.title
                )
                has_changed = True
            except requests.exceptions.RequestException:
                created = False

            if created:
                # Pre-generate renditions for whichever of portrait, square and landscape poster images
                # have been supplied, indexed under PORTRAIT_IMAGE_FILTER, SQUARE_IMAGE_FILTER
                # and LANDSCAPE_IMAGE_FILTER respectively so that when we look those up again we get
                # the original appropriately-cropped version
                portrait_filter = Filter(self.PORTRAIT_IMAGE_FILTER)
                self.poster_image.renditions.create(
                    filter_spec=self.PORTRAIT_IMAGE_FILTER,
                    file=portrait_image_file,
                    focal_point_key=portrait_filter.get_cache_key(self.poster_image)
                )

                if self.poster_square_src_original:
                    square_filter = Filter(self.SQUARE_IMAGE_FILTER)
                    try:
                        self.poster_image.renditions.create(
                            filter_spec=self.SQUARE_IMAGE_FILTER,
                            file=self._image_file_from_url(
                                urljoin(self.original_url, self.poster_square_src_original)
                            ),
                            focal_point_key=square_filter.get_cache_key(self.poster_image)
                        )
                    except requests.exceptions.RequestException:
                        pass

                if self.poster_landscape_src_original:
                    landscape_filter = Filter(self.LANDSCAPE_IMAGE_FILTER)
                    try:
                        self.poster_image.renditions.create(
                            filter_spec=self.LANDSCAPE_IMAGE_FILTER,
                            file=self._image_file_from_url(
                                urljoin(self.original_url, self.poster_landscape_src_original)
                            ),
                            focal_point_key=landscape_filter.get_cache_key(self.poster_image)
                        )
                    except requests.exceptions.RequestException:
                        pass

        return has_changed

    def _import_content_images(self):
        new_pages = []
        has_changed = False

        for page in self.pages:
            if not isinstance(page.block, PageBlock):
                # leave unrecognised block types unchanged in the output
                new_pages.append((page.block_type, page.value))
                continue

            page_html = page.value['html'].source
            page_dom = dom = BeautifulSoup(page_html, 'html.parser')
            # look for <amp-img> elements with src attributes
            for img_tag in page_dom.select('amp-img[src]'):
                image_url = urljoin(self.original_url, img_tag['src'])
                title = (
                    img_tag.get('alt')
                    or _name_from_url(image_url)
                    or ("image from story: %s" % self.title)
                )
                try:
                    image, created = self._image_from_url(image_url, title=title)
                    img_tag['data-wagtail-image-id'] = image.id
                    del img_tag['src']
                    has_changed = True
                except requests.exceptions.RequestException:
                    pass

            page.value['html'] = AMPText(str(page_dom))
            new_pages.append((page.block_type, page.value))

        if has_changed:
            self.pages = new_pages

        return has_changed

    def _image_file_from_url(self, url):
        url_obj = urlparse(url)
        filename = url_obj.path.split('/')[-1] or 'image'
        response = requests.get(url)
        response.raise_for_status()
        return ImageFile(ContentFile(response.content), name=filename)

    def _create_image(self, file, title=None):
        """
        Return an unsaved image instance for the given ImageFile and title. Override this if your
        image model requires extra metadata to be filled in, or you want to assign it to a specific
        collection
        """
        return Image(file=file, title=title)

    def _image_from_image_file(self, file, title=None):
        with file.open() as f:
            file_hash = hashlib.sha1(f.read()).hexdigest()

        # check if we already have an image with the same file content
        image = Image.objects.filter(file_hash=file_hash).first()
        if image:
            return (image, False)
        else:
            image = self._create_image(file, title=title)
            image.save()
            image.get_file_hash()  # ensure file_hash is populated
            return (image, True)

    def _image_from_url(self, url, title=None):
        image_file = self._image_file_from_url(url)
        return self._image_from_image_file(image_file, title=title)

    def import_videos(self):
        # if flag indicates we have imported videos on this instance already,
        # don't repeat; this allows us to call import_videos / save within a
        # post_save signal without the second save retriggering a full import_videos
        if getattr(self, '_has_imported_videos', False):
            return False  # report no changes

        content_has_changed = False
        new_pages = []

        for page in self.pages:
            if not isinstance(page.block, PageBlock):
                # leave unrecognised block types unchanged in the output
                new_pages.append((page.block_type, page.value))
                continue

            page_html = page.value['html'].source
            page_dom = dom = BeautifulSoup(page_html, 'html.parser')
            # look for <amp-video> elements
            for video_tag in page_dom.select('amp-video'):
                poster_url = video_tag.get('poster')
                if poster_url:
                    try:
                        poster_image_file = self._image_file_from_url(
                            urljoin(self.original_url, poster_url)
                        )
                    except requests.exceptions.RequestException:
                        poster_image_file = None
                else:
                    poster_image_file = None

                width = video_tag.get('width')
                height = video_tag.get('height')
                fallback_title = "video from story: %s" % self.title

                video_url = video_tag.get('src')
                if video_url:
                    title = _name_from_url(video_url) or fallback_title
                    try:
                        video, created = self._video_from_url(
                            urljoin(self.original_url, video_url),
                            title=title, width=width, height=height,
                            thumbnail=poster_image_file
                        )
                        video_tag['data-wagtail-media-id'] = video.id
                        del video_tag['src']
                        content_has_changed = True
                    except requests.exceptions.RequestException:
                        pass

                for source_tag in video_tag.find_all('source', recursive=False):
                    video_url = source_tag.get('src')
                    if video_url:
                        title = _name_from_url(video_url) or fallback_title
                        try:
                            video, created = self._video_from_url(
                                urljoin(self.original_url, video_url),
                                title=title, width=width, height=height,
                                thumbnail=poster_image_file
                            )
                            source_tag['data-wagtail-media-id'] = video.id
                            del source_tag['src']
                            content_has_changed = True
                        except requests.exceptions.RequestException:
                            pass

            page.value['html'] = AMPText(str(page_dom))
            new_pages.append((page.block_type, page.value))

        if content_has_changed:
            self.pages = new_pages

        self._has_imported_videos = True
        return content_has_changed

    def _video_file_from_url(self, url):
        url_obj = urlparse(url)
        filename = url_obj.path.split('/')[-1] or 'video'
        response = requests.get(url)
        response.raise_for_status()
        return File(ContentFile(response.content), name=filename)

    def _create_video(self, file, **kwargs):
        from wagtailmedia.models import get_media_model
        Media = get_media_model()
        # for wagtailmedia < 0.7, need to ensure that the duration field
        # is populated
        if 'duration' not in kwargs and not Media._meta.get_field('duration').blank:
            kwargs['duration'] = 0
        video = Media(file=file, **kwargs)
        return video

    def _video_from_url(self, url, **kwargs):
        video_file = self._video_file_from_url(url)
        video = self._create_video(video_file, type='video', **kwargs)
        video.save()
        return (video, True)

    class Meta:
        abstract = True


class BaseWebStoryPage(WebStoryPageMixin, Page):
    content_panels = Page.content_panels + WebStoryPageMixin.web_story_content_panels
    promote_panels = Page.promote_panels + WebStoryPageMixin.web_story_promote_panels

    class Meta:
        abstract = True


def get_story_page_models():
    """
    Return a list of all non-abstract page models that inherit from WebStoryPageMixin
    """
    return [
        model for model in get_page_models()
        if issubclass(model, WebStoryPageMixin)
    ]


class ExternalStory(models.Model):
    url = models.TextField()
    # a SHA-1 hash of the URL
    url_hash = models.CharField(max_length=40, editable=False, unique=True, db_index=True)
    title = models.TextField(blank=True, editable=False)
    publisher = models.TextField(blank=False, editable=False)
    publisher_logo_src = models.TextField('Publisher logo URL', blank=True, editable=False)
    poster_portrait_src = models.TextField('Poster portrait image URL', blank=True, editable=False)
    poster_square_src = models.TextField('Poster square image URL', blank=True, editable=False)
    poster_landscape_src = models.TextField('Poster landscape image URL', blank=True, editable=False)
    last_fetched_at = models.DateTimeField()

    @classmethod
    def get_for_url(cls, url):
        url_hash = hashlib.sha1(url.encode('utf-8')).hexdigest()
        try:
            return cls.objects.get(url_hash=url_hash)
        except cls.DoesNotExist:
            html = requests.get(url).text
            story = Story(html)
            result, created = cls.objects.update_or_create(
                url_hash=url_hash, defaults={
                    'url': url,
                    'title': story.title,
                    'publisher': story.publisher,
                    'publisher_logo_src': urljoin(url, story.publisher_logo_src) if story.publisher_logo_src else '',
                    'poster_portrait_src': urljoin(url, story.poster_portrait_src) if story.poster_portrait_src else '',
                    'poster_square_src': urljoin(url, story.poster_square_src) if story.poster_square_src else '',
                    'poster_landscape_src': urljoin(url, story.poster_landscape_src) if story.poster_landscape_src else '',
                    'last_fetched_at': timezone.now(),
                }
            )
            return result



# Generated by Django 2.2.17 on 2021-05-27 10:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_webstories', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='externalstory',
            name='url_hash',
            field=models.CharField(db_index=True, editable=False, max_length=40, unique=True),
        ),
    ]



from django.conf import settings
from django.urls import path, include, reverse
from wagtail.admin.menu import MenuItem
from wagtail.core import hooks

from . import admin_urls

if getattr(settings, 'WAGTAIL_WEBSTORIES_IMPORT_MODEL', None):

    @hooks.register('register_admin_urls')
    def register_admin_urls():
        return [
            path('webstories/', include(admin_urls, namespace='wagtail_webstories')),
        ]


    @hooks.register('register_admin_menu_item')
    def register_webstories_item():
        return MenuItem('Web stories', reverse('wagtail_webstories:import_story'), classnames='icon icon-openquote', order=10000)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="wagtail-webstories",
    version="0.0.4",
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    test_suite="tests",
    url="https://github.com/torchbox/wagtail-webstories/",

    author="Matt Westcott",
    author_email="matthew@torchbox.com",
    description="AMP web story support for Wagtail",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Framework :: Wagtail :: 2",
        "Topic :: Internet :: WWW/HTTP",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        "webstories>=0.0.1,<0.1",
        "requests>=2.24.0,<3",
        "beautifulsoup4>=4.6,<5",
    ],
    extras_require={
        "testing": [
            "responses>=0.12,<1",
            "Pillow>=4.0.0,<9.0.0",
            "wagtailmedia>=0.6,<0.8",
        ]
    },
    license="BSD",
)



import re
from django.apps import apps
from django.utils.html import escape
from django.utils.safestring import mark_safe
from wagtail.images import get_image_model_string
from wagtail.images.shortcuts import get_rendition_or_not_found

# Retrieve the (possibly custom) image model. Can't use get_image_model as of Wagtail 2.11, as we
# need require_ready=False: https://github.com/wagtail/wagtail/pull/6568
Image = apps.get_model(get_image_model_string(), require_ready=False)


def _replace_image_id(match):
    try:
        image = Image.objects.get(id=match.group(1))
    except Image.DoesNotExist:
        return ''

    rendition = get_rendition_or_not_found(image, 'original')
    return 'src="%s"' % escape(rendition.url)


def _replace_media_id(match):
    from wagtailmedia.models import get_media_model
    Media = get_media_model()

    try:
        media = Media.objects.get(id=match.group(1))
    except Media.DoesNotExist:
        return ''

    return 'src="%s"' % escape(media.url)


FIND_DATA_WAGTAIL_IMAGE_ID_ATTR = re.compile(r'''\bdata-wagtail-image-id=["'](\d+)["']''')
FIND_DATA_WAGTAIL_MEDIA_ID_ATTR = re.compile(r'''\bdata-wagtail-media-id=["'](\d+)["']''')

def expand_entities(html):
    """
    Expand symbolic references in a string of AMP markup - e.g. convert
    data-wagtail-image-id="123" to src="/path/to/image.html"
    """
    html = FIND_DATA_WAGTAIL_IMAGE_ID_ATTR.sub(_replace_image_id, html)
    html = FIND_DATA_WAGTAIL_MEDIA_ID_ATTR.sub(_replace_media_id, html)
    return html


class AMPText:
    """Equivalent of Wagtail's RichText - performs entity expansion when rendered"""
    def __init__(self, source):
        self.source = (source or '')

    def __html__(self):
        return expand_entities(self.source)

    def __str__(self):
        return mark_safe(self.__html__())

    def __bool__(self):
        return bool(self.source)



from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _
from wagtail.admin.widgets import AdminPageChooser
from wagtail.core.models import Page

class ImportStoryForm(forms.Form):
    source_url = forms.URLField(required=True)
    destination = forms.ModelChoiceField(
        required=True,
        queryset=Page.objects.all(),
        widget=AdminPageChooser(can_choose_root=True, user_perms='copy_to'),
        help_text=_("Where the new story page will be created")
    )

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user')
        super().__init__(*args, **kwargs)

    def clean_destination(self):
        destination = self.cleaned_data['destination']
        if not destination.permissions_for_user(self.user).can_add_subpage():
            raise ValidationError(_("You do not have permission to create a page at this location"))
        return destination
